@@ -1,5 +1,6 @@
 /*********************************************************
- * Copyright (C) 2006-2015 VMware, Inc. All rights reserved.
+ * Copyright (c) 2006-2024 Broadcom. All Rights Reserved.
+ * The term "Broadcom" refers to Broadcom Inc. and/or its subsidiaries.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -20,7 +21,7 @@
 /*
  * vmcheck.c --
  *
- *    Utility functions for discovering our virtualization status.
+ *      Utility functions for discovering our virtualization status.
  */
 
 #include <stdlib.h>
@@ -30,11 +31,19 @@
 #   include <ntddk.h>
 #endif
 
+#if (!defined(WINNT_DDK) && defined(_WIN32))
+// include windows.h, otherwise DWORD type used in hostinfo.h is not defined
+#  include "windows.h"
+#endif
+
 #include "vmware.h"
 #include "vm_version.h"
 #include "vm_tools_version.h"
+
 #if !defined(WINNT_DDK)
 #  include "hostinfo.h"
+#  include "str.h"
+#  include "x86cpuid.h"
 #endif
 
 /*
@@ -56,13 +65,15 @@
 #include "backdoor_def.h"
 #include "debug.h"
 
-
-typedef Bool (*SafeCheckFn)(void);
-
 #if !defined(_WIN32)
 #   include "vmsignal.h"
 #   include "setjmp.h"
+#endif
 
+typedef Bool (*SafeCheckFn)(void);
+
+
+#if !defined(_WIN32)
 static sigjmp_buf jmpBuf;
 static Bool       jmpIsSet;
 
@@ -134,6 +145,9 @@ VmCheckSafe(SafeCheckFn checkFn)
 #else
    do {
       int signals[] = {
+#if defined(__FreeBSD__)
+         SIGBUS,
+#endif
          SIGILL,
          SIGSEGV,
       };
@@ -184,7 +198,7 @@ VmCheckSafe(SafeCheckFn checkFn)
 
 Bool
 VmCheck_GetVersion(uint32 *version, // OUT
-                    uint32 *type)    // OUT
+                   uint32 *type)    // OUT
 {
    Backdoor_proto bp;
 
@@ -192,7 +206,7 @@ VmCheck_GetVersion(uint32 *version, // OUT
    ASSERT(type);
 
    /* Make sure EBX does not contain BDOOR_MAGIC */
-   bp.in.size = ~BDOOR_MAGIC;
+   bp.in.size = (size_t)~BDOOR_MAGIC;
    /* Make sure ECX does not contain any known VMX type */
    bp.in.cx.halfs.high = 0xFFFF;
 
@@ -230,6 +244,53 @@ VmCheck_GetVersion(uint32 *version, // OUT
 }
 
 
+/*
+ *----------------------------------------------------------------------
+ *
+ * VmCheck_GetHWVersion --
+ *
+ *    Retrieve the VM's hardware version.
+ *
+ * Return value:
+ *    TRUE on success
+ *       *hwversion contains the VM hardware version
+ *    FALSE on failure
+ *
+ * Side effects:
+ *    None
+ *
+ *----------------------------------------------------------------------
+ */
+
+Bool
+VmCheck_GetHWVersion(uint32 *hwversion) // OUT
+{
+   Backdoor_proto bp;
+   ASSERT(hwversion);
+
+   /* Make sure EBX does not contain BDOOR_MAGIC */
+   bp.in.size = (size_t)~BDOOR_MAGIC;
+   /* send backdoor command to get hw version */
+   bp.in.cx.halfs.low = BDOOR_CMD_GETHWVERSION;
+   Backdoor(&bp);
+
+   if (bp.out.ax.word == 0xFFFFFFFF) {
+      /*
+       * No backdoor device there. This code is not executing in a VMware
+       * virtual machine.
+       */
+      return FALSE;
+   }
+
+   if (bp.out.bx.word != BDOOR_MAGIC) {
+      return FALSE;
+   }
+
+   *hwversion = bp.out.ax.word;
+   return TRUE;
+}
+
+
 /*
  *----------------------------------------------------------------------
  *
@@ -239,7 +300,8 @@ VmCheck_GetVersion(uint32 *version, // OUT
  *    environment.
  *
  * Return value:
- *    TRUE if we're in a virtual machine, FALSE otherwise.
+ *    TRUE if we're in a virtual machine or a Linux compilation using Valgrind,
+ *    FALSE otherwise.
  *
  * Side effects:
  *    None.
@@ -254,15 +316,58 @@ VmCheck_IsVirtualWorld(void)
    uint32 dummy;
 
 #if !defined(WINNT_DDK)
-   if (VmCheckSafe(Hostinfo_TouchXen)) {
-      Debug("%s: detected Xen.\n", __FUNCTION__);
-      return FALSE;
-   }
+#ifdef USE_VALGRIND
+   /*
+    * Valgrind can't handle the backdoor check.
+    */
+   return TRUE;
+#endif
+#if defined VM_X86_ANY
+   char *hypervisorSig;
+   uint32 i;
 
-   if (VmCheckSafe(Hostinfo_TouchVirtualPC)) {
-      Debug("%s: detected Virtual PC.\n", __FUNCTION__);
-      return FALSE;
+   /*
+    * Check for other environments like Xen and VirtualPC only if we haven't
+    * already detected that we are on a VMware hypervisor. See PR 1035346.
+    */
+   hypervisorSig = Hostinfo_HypervisorCPUIDSig();
+   if (hypervisorSig == NULL ||
+         Str_Strcmp(hypervisorSig, CPUID_VMWARE_HYPERVISOR_VENDOR_STRING) != 0) {
+      if (hypervisorSig != NULL) {
+         static const struct {
+            const char *vendorSig;
+            const char *hypervisorName;
+         } hvVendors[] = {
+            { CPUID_KVM_HYPERVISOR_VENDOR_STRING, "Linux KVM" },
+            { CPUID_XEN_HYPERVISOR_VENDOR_STRING, "Xen" },
+         };
+
+         for (i = 0; i < ARRAYSIZE(hvVendors); i++) {
+            if (Str_Strcmp(hypervisorSig, hvVendors[i].vendorSig) == 0) {
+               Debug("%s: detected %s.\n", __FUNCTION__,
+                     hvVendors[i].hypervisorName);
+               free(hypervisorSig);
+               return FALSE;
+            }
+         }
+      }
+
+      free(hypervisorSig);
+
+      if (VmCheckSafe(Hostinfo_TouchXen)) {
+         Debug("%s: detected Xen.\n", __FUNCTION__);
+         return FALSE;
+      }
+
+      if (VmCheckSafe(Hostinfo_TouchVirtualPC)) {
+         Debug("%s: detected Virtual PC.\n", __FUNCTION__);
+         return FALSE;
+      }
+
+   } else {
+      free(hypervisorSig);
    }
+#endif
 
    if (!VmCheckSafe(Hostinfo_TouchBackDoor)) {
       Debug("%s: backdoor not detected.\n", __FUNCTION__);
@@ -270,14 +375,20 @@ VmCheck_IsVirtualWorld(void)
    }
 
    /* It should be safe to use the backdoor without a crash handler now. */
-   VmCheck_GetVersion(&version, &dummy);
+   if (!VmCheck_GetVersion(&version, &dummy)) {
+      Debug("%s: VmCheck_GetVersion failed.\n", __FUNCTION__);
+      return FALSE;
+   }
 #else
    /*
     * The Win32 vmwvaudio driver uses this function, so keep the old,
     * VMware-only check.
     */
    __try {
-      VmCheck_GetVersion(&version, &dummy);
+      if (!VmCheck_GetVersion(&version, &dummy)) {
+         Debug("%s: VmCheck_GetVersion failed.\n", __FUNCTION__);
+         return FALSE;
+      }
    } __except (GetExceptionCode() == STATUS_PRIVILEGED_INSTRUCTION) {
       return FALSE;
    }
@@ -285,8 +396,9 @@ VmCheck_IsVirtualWorld(void)
 
    if (version != VERSION_MAGIC) {
       Debug("The version of this program is incompatible with your %s.\n"
-            "For information on updating your VMware Tools please see\n"
-            "http://www.vmware.com/info?id=99\n"
+            "For information on updating your VMware Tools please see the\n"
+            "'Upgrading VMware Tools' section of the 'VMware Tools User Guide'"
+            "\nat https://docs.vmware.com/en/VMware-Tools/index.html\n"
             "\n", PRODUCT_LINE_NAME);
       return FALSE;
    }
