@@ -1,5 +1,6 @@
 /*********************************************************
- * Copyright (C) 2011-2015 VMware, Inc. All rights reserved.
+ * Copyright (c) 2011-2019,2024 Broadcom. All Rights Reserved.
+ * The term "Broadcom" refers to Broadcom Inc. and/or its subsidiaries.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -30,6 +31,7 @@
 #include <errno.h>
 #include <locale.h>
 #include <stdio.h>
+#include <stdlib.h>
 #include <string.h>
 #include <glib.h>
 #ifdef _WIN32
@@ -52,7 +54,7 @@ typedef struct MsgCatalog {
 
 typedef struct MsgState {
    GHashTable *domains;  /* List of text domains. */
-   GStaticMutex lock;    /* Mutex to protect shared state. */
+   GMutex lock;          /* Mutex to protect shared state. */
 } MsgState;
 
 
@@ -367,7 +369,7 @@ MsgInitState(gpointer unused)
 {
    ASSERT(msgState == NULL);
    msgState = g_new0(MsgState, 1);
-   g_static_mutex_init(&msgState->lock);
+   g_mutex_init(&msgState->lock);
    return NULL;
 }
 
@@ -463,7 +465,19 @@ MsgGetUserLanguage(void)
     * POSIX implementation: just use setlocale() to query the data. Ignore any
     * codeset information.
     */
-   char *tmp = setlocale(LC_MESSAGES, NULL);
+   char *tmp;
+   /*
+    * This is useful for testing, and also seems to be used by some
+    * distros (NeoKylin) rather than the setlocale() APIs.
+    * See PR 1672149
+    */
+   char *envLocale = getenv("LANG");
+   if (envLocale != NULL) {
+      lang = g_strdup(envLocale);
+      g_debug("%s: Using LANG override of '%s'\n", __FUNCTION__, lang);
+      return lang;
+   }
+   tmp = setlocale(LC_MESSAGES, NULL);
    if (tmp == NULL) {
       lang = g_strdup("C");
    } else {
@@ -519,22 +533,8 @@ MsgSetCatalog(const char *domain,
  * MsgLoadCatalog --                                                     */ /**
  *
  * Loads the message catalog at the given path into a new hash table.
- *
- * This function supports an "extended" format for the catalog files. Aside
- * from the usual things you can put in a lib/dict-based dictionary, this code
- * supports multi-line messages so that long messages can be broken down.
- *
- * These lines are any lines following a key / value declaration that start with
- * a quote character (ignoring any leading spaces and tabs). So a long message
- * could look like this:
- *
- * @code
- * message.id = "This is the first part of the message. "
- *              "This is the continuation line, still part of the same message."
- * @endcode
- *
- * The complete value for the "message.id" key will be the concatenation of
- * the values in quotes.
+ * The catalog entries are a simple <key> = <value>.  Line continutatiom
+ * is not supported.
  *
  * @param[in] path    Path containing the message catalog (encoding should be
  *                    UTF-8).
@@ -558,6 +558,7 @@ MsgLoadCatalog(const char *path)
    ASSERT(localPath != NULL);
 
    stream = g_io_channel_new_file(localPath, "r", &err);
+   g_debug("%s: loading message catalog '%s'\n", __FUNCTION__, localPath);
    RELEASE_FILENAME_LOCAL(localPath);
 
    if (err != NULL) {
@@ -571,90 +572,58 @@ MsgLoadCatalog(const char *path)
                                 g_free,
                                 g_free);
    for (;;) {
-      gboolean eof = FALSE;
       char *name = NULL;
       char *value = NULL;
       gchar *line;
+      gsize len;
+      gsize term;
+      char *unused = NULL;
 
       /* Read the next key / value pair. */
-      for (;;) {
-         gsize i;
-         gsize len;
-         gsize term;
-         char *unused = NULL;
-         gboolean cont = FALSE;
 
-         g_io_channel_read_line(stream, &line, &len, &term, &err);
-
-         if (err != NULL) {
-            g_warning("Unable to read a line from '%s': %s\n",
-                      path, err->message);
-            g_clear_error(&err);
-            error = TRUE;
-            g_free(line);
-            break;
-         }
-
-         if (line == NULL) {
-            eof = TRUE;
-            break;
-         }
-
-         /*
-          * Fix the line break to always be Unix-style, to make lib/dict
-          * happy.
-          */
-         if (line[term] == '\r') {
-            line[term] = '\n';
-            if (len > term) {
-               line[term + 1] = '\0';
-            }
-         }
-
-         /*
-          * If currently name is not NULL, then check if this is a continuation
-          * line and, if it is, just append the contents to the current value.
-          */
-         if (name != NULL && term > 0 && line[term - 1] == '"') {
-            for (i = 0; i < len; i++) {
-               if (line[i] == '"') {
-                  /* OK, looks like a continuation line. */
-                  char *tmp;
-
-                  line[term - 1] = '\0';
-                  tmp = g_strdup_printf("%s%s", value, line + i + 1);
-                  g_free(value);
-                  value = tmp;
-                  cont = TRUE;
-                  break;
-               } else if (line[i] != ' ' && line[i] != '\t') {
-                  break;
-               }
-            }
-         }
-
-         /*
-          * If not a continuation line and we have a name, break out of the
-          * inner loop to update the dictionaty.
-          */
-         if (!cont && name != NULL) {
-            g_free(line);
-            break;
-         }
-
-         /*
-          * Finally, try to parse the string using the dictionary library.
-          */
-         if (!cont && DictLL_UnmarshalLine(line, len, &unused, &name, &value) == NULL) {
-            g_warning("Couldn't parse line from catalog: %s", line);
-            error = TRUE;
-         }
+      g_io_channel_read_line(stream, &line, &len, &term, &err);
 
+      if (err != NULL) {
+         g_warning("Unable to read a line from '%s': %s\n",
+                   path, err->message);
+         g_clear_error(&err);
+         error = TRUE;
          g_free(line);
-         g_free(unused);
+         break;
       }
 
+      if (line == NULL) {
+         /* This signifies EOF. */
+         break;
+      }
+
+      /*
+       * Fix the line break to always be Unix-style, to make lib/dict
+       * happy.
+       */
+      if (line[term] == '\r') {
+         line[term] = '\n';
+         if (len > term) {
+            line[term + 1] = '\0';
+         }
+      }
+
+      /*
+       * Try to parse the string using the dictionary library.
+       */
+      if (DictLL_UnmarshalLine(line, len, &unused, &name, &value) == NULL) {
+         g_warning("Couldn't parse line from catalog: %s", line);
+         error = TRUE;
+      }
+      g_free(unused);
+      g_free(line);
+
       if (error) {
+         /*
+          * If the local DictLL_UnmarshalLine() returns NULL, name and value
+          * will remain NULL pointers.  No malloc'ed memory to free here.
+          */
+         /* coverity[leaked_storage] */
          break;
       }
 
@@ -666,6 +635,8 @@ MsgLoadCatalog(const char *path)
              !g_utf8_validate(value, -1, NULL)) {
             g_warning("Invalid UTF-8 string in message catalog (key = %s)\n", name);
             error = TRUE;
+            g_free(name);
+            g_free(value);
             break;
          }
 
@@ -673,14 +644,8 @@ MsgLoadCatalog(const char *path)
          val = g_strcompress(value);
          g_free(value);
 
-         // the hashtable takes ownership of the memory for 'name' and 'value'
+         // the hashtable takes ownership of the memory for 'name' and 'val'
          g_hash_table_insert(dict, name, val);
-         name = NULL;
-         value = NULL;
-      }
-
-      if (eof) {
-         break;
       }
    }
 
@@ -735,6 +700,8 @@ I18n_BindTextDomain(const char *domain,
    if (lang == NULL || *lang == '\0') {
       usrlang = MsgGetUserLanguage();
       lang = usrlang;
+   } else {
+      usrlang = g_strdup(lang);
    }
 
    /*
@@ -747,16 +714,27 @@ I18n_BindTextDomain(const char *domain,
    file = g_strdup_printf("%s%smessages%s%s%s%s.vmsg",
                           catdir, DIRSEPS, DIRSEPS, lang, DIRSEPS, domain);
 
+   /*
+    * If we couldn't find the catalog file for the user's language, see if
+    * there's an encoding to chop off first, eg zh_CN.UTF-8
+    */
    if (!g_file_test(file, G_FILE_TEST_IS_REGULAR)) {
-      /*
-       * If we couldn't find the catalog file for the user's language, see if
-       * we can find a more generic language (e.g., for "en_US", also try "en").
-       */
-      char *sep = strrchr(lang, '_');
+      const char *sep = strrchr(lang, '.');
+      if (sep != NULL) {
+         usrlang[sep - lang] = '\0';
+         g_free(file);
+         file = g_strdup_printf("%s%smessages%s%s%s%s.vmsg",
+                                catdir, DIRSEPS, DIRSEPS, usrlang, DIRSEPS, domain);
+      }
+   }
+
+   /*
+    * If we couldn't find the catalog file for the user's language, see if
+    * we can find a more generic language (e.g., for "en_US", also try "en").
+    */
+   if (!g_file_test(file, G_FILE_TEST_IS_REGULAR)) {
+      const char *sep = strrchr(lang, '_');
       if (sep != NULL) {
-         if (usrlang == NULL) {
-            usrlang = g_strdup(lang);
-         }
          usrlang[sep - lang] = '\0';
          g_free(file);
          file = g_strdup_printf("%s%smessages%s%s%s%s.vmsg",
@@ -776,9 +754,9 @@ I18n_BindTextDomain(const char *domain,
                    "catalog dir '%s'.\n", domain, lang, catdir);
       }
    } else {
-      g_static_mutex_lock(&state->lock);
+      g_mutex_lock(&state->lock);
       MsgSetCatalog(domain, catalog);
-      g_static_mutex_unlock(&state->lock);
+      g_mutex_unlock(&state->lock);
    }
    g_free(file);
    g_free(usrlang);
@@ -833,7 +811,7 @@ I18n_GetString(const char *domain,
    memcpy(idBuf, idp, len);
    idBuf[len] = '\0';
 
-   g_static_mutex_lock(&state->lock);
+   g_mutex_lock(&state->lock);
 
    catalog = MsgGetCatalog(domain);
    if (catalog != NULL) {
@@ -841,15 +819,13 @@ I18n_GetString(const char *domain,
    }
 
    if (source != NULL) {
-      const void *retval = NULL;
-
-      retval = g_hash_table_lookup(source, idBuf);
+      const void *retval = g_hash_table_lookup(source, idBuf);
       if (NULL != retval) {
          strp = retval;
       }
    }
 
-   g_static_mutex_unlock(&state->lock);
+   g_mutex_unlock(&state->lock);
 
    return strp;
 }
