@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2013 VMware, Inc. All rights reserved.
+ * Copyright (c) 2013,2021 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -22,10 +22,6 @@
  * Main entry points for fuse file operations for HGFS
  */
 
-#if FUSE_USE_VERSION >= 26
-#define HAVE_UTIMENSAT 1
-#endif
-
 #include "module.h"
 #include "cache.h"
 #include "filesystem.h"
@@ -119,9 +115,16 @@ freeAbsPath(char *abspath)  // IN
  *----------------------------------------------------------------------
  */
 
+#if FUSE_MAJOR_VERSION == 3
+static int
+hgfs_getattr(const char *path,           //IN: path of a file/directory
+             struct stat *stbuf,         //IN/OUT: file/directoy attribute
+             struct fuse_file_info *fi)  //IN/OUT: Unused
+#else
 static int
 hgfs_getattr(const char *path,    //IN: path of a file/directory
              struct stat *stbuf)  //IN/OUT: file/directoy attribute
+#endif
 {
    HgfsHandle fileHandle = HGFS_INVALID_HANDLE;
    HgfsAttrInfo newAttr = {0};
@@ -333,9 +336,10 @@ hgfs_readlink(const char *path, //IN: path to a file
       goto exit;
    }
 
-   if (size >= strlen(attr->fileName)) {
-      Str_Strcpy(buf, attr->fileName + gState->basePathLen,
-                 strlen(attr->fileName) + 1 - gState->basePathLen);
+   if (size > strlen(attr->fileName)) {
+      Str_Strcpy(buf, attr->fileName,
+                 strlen(attr->fileName) + 1);
+      LOG(4, ("ReadLink: link target name = %s\n", buf));
    } else {
       res = -ENOBUFS;
    }
@@ -364,12 +368,22 @@ exit:
  *----------------------------------------------------------------------
  */
 
+#if FUSE_MAJOR_VERSION == 3
+static int
+hgfs_readdir(const char *path,              //IN: path to a directory
+             void *buf,                     //OUT: buffer to fill the dir entry
+             fuse_fill_dir_t filler,        //IN: function pointer to fill buf
+             off_t offset,                  //IN: offset to read the dir
+             struct fuse_file_info *fi,     //IN: file info set by open call
+             enum fuse_readdir_flags flags) //IN: unused
+#else
 static int
 hgfs_readdir(const char *path,          //IN: path to a directory
              void *buf,                 //OUT: buffer to fill the dir entry
              fuse_fill_dir_t filler,    //IN: function pointer to fill buf
              off_t offset,              //IN: offset to read the dir
              struct fuse_file_info *fi) //IN: file info set by open call
+#endif
 {
    char *abspath = NULL;
    int res = 0;
@@ -495,7 +509,9 @@ hgfs_unlink(const char *path) //IN: path to a file
    }
 
    res = HgfsDelete(abspath, HGFS_OP_DELETE_FILE);
-   HgfsInvalidateAttrCache(abspath);
+   if (res == 0) {
+      HgfsInvalidateAttrCache(abspath);
+   }
 
 exit:
    LOG(4, ("Exit(%d)\n", res));
@@ -533,7 +549,9 @@ hgfs_rmdir(const char *path) //IN: path to a dir
    }
 
    res = HgfsDelete(abspath, HGFS_OP_DELETE_DIR);
-   HgfsInvalidateAttrCache(abspath);
+   if (res == 0) {
+      HgfsInvalidateAttrCache(abspath);
+   }
 
 exit:
    LOG(4, ("Exit(%d)\n", res));
@@ -559,29 +577,24 @@ exit:
  */
 
 static int
-hgfs_symlink(const char *from,      //IN: from path
-             const char *to)        //IN: to path
+hgfs_symlink(const char *symname,   //IN: symname target
+             const char *source)    //IN: source name
 {
-   char *absfrom = NULL;
-   char *absto = NULL;
+   char *absSource = NULL;
    int res;
 
-   LOG(4, ("Entry(from = %s, to = %s)\n", from, to));
-   res = getAbsPath(from, &absfrom);
-   if (res < 0) {
-      goto exit;
-   }
-   res = getAbsPath(to, &absto);
+   LOG(4, ("Entry(from = %s, to = %s)\n", symname, source));
+   res = getAbsPath(source, &absSource);
    if (res < 0) {
       goto exit;
    }
 
-   res = HgfsSymlink(absto, absfrom);
+   LOG(4, ("symname = %s, abs source = %s)\n", symname, absSource));
+   res = HgfsSymlink(absSource, symname);
 
 exit:
    LOG(4, ("Exit(%d)\n", res));
-   freeAbsPath(absfrom);
-   freeAbsPath(absto);
+   freeAbsPath(absSource);
    return res;
 }
 
@@ -602,9 +615,16 @@ exit:
  *----------------------------------------------------------------------
  */
 
+#if FUSE_MAJOR_VERSION == 3
+static int
+hgfs_rename(const char *from,    //IN: from path name
+            const char *to,      //IN: to path name
+            unsigned int flags)  //IN: unused
+#else
 static int
 hgfs_rename(const char *from,  //IN: from path name
             const char *to)    //IN: to path name
+#endif
 {
    char *absfrom = NULL;
    char *absto = NULL;
@@ -621,7 +641,10 @@ hgfs_rename(const char *from,  //IN: from path name
    }
 
    res = HgfsRename(absfrom, absto);
-   HgfsInvalidateAttrCache(absfrom);
+   if (res == 0) {
+      HgfsInvalidateAttrCache(absfrom);
+      HgfsInvalidateAttrCache(absto);
+   }
 
 exit:
    LOG(4, ("Exit(%d)\n", res));
@@ -692,9 +715,16 @@ exit:
  *----------------------------------------------------------------------
  */
 
+#if FUSE_MAJOR_VERSION == 3
+static int
+hgfs_chmod(const char *path,          //IN: path to a file
+           mode_t mode,               //IN: mode to set
+           struct fuse_file_info *fi) //IN/OUT: unused
+#else
 static int
 hgfs_chmod(const char *path,   //IN: path to a file
            mode_t mode)        //IN: mode to set
+#endif
 {
    char *abspath = NULL;
    int res;
@@ -757,10 +787,18 @@ exit:
  *----------------------------------------------------------------------
  */
 
+#if FUSE_MAJOR_VERSION == 3
+static int
+hgfs_chown(const char *path,           //IN: Path to a file
+           uid_t uid,                  //IN: User id
+           gid_t gid,                  //IN: Group id
+           struct fuse_file_info *fi)  //IN/OUT: unused
+#else
 static int
 hgfs_chown(const char *path,  //IN: Path to a file
            uid_t uid,         //IN: User id
            gid_t gid)         //IN: Group id
+#endif
 {
    HgfsHandle fileHandle = HGFS_INVALID_HANDLE;
    HgfsAttrInfo newAttr = {0};
@@ -820,9 +858,16 @@ exit:
  *----------------------------------------------------------------------
  */
 
+#if FUSE_MAJOR_VERSION == 3
+static int
+hgfs_truncate(const char *path,           //IN: path to a file
+              off_t size,                 //IN: new size
+              struct fuse_file_info *fi)  //IN/OUT: unused
+#else
 static int
 hgfs_truncate(const char *path,  //IN: path to a file
               off_t size)        //IN: new size
+#endif
 {
    HgfsHandle fileHandle = HGFS_INVALID_HANDLE;
    HgfsAttrInfo newAttr = {0};
@@ -881,14 +926,15 @@ exit:
  *----------------------------------------------------------------------
  */
 
-#ifdef HAVE_UTIMENSAT
+#if FUSE_MAJOR_VERSION == 3
+static int
+hgfs_utimens(const char *path,              //IN: path to a file
+             const struct timespec ts[2],   //IN: new time
+             struct fuse_file_info *fi)     //IN/OUT: unused
+#else
 static int
 hgfs_utimens(const char *path,              //IN: path to a file
              const struct timespec ts[2])   //IN: new time
-#else
-static int
-hgfs_utime(const char *path,        //IN: path to a file
-           struct utimbuf *times)   //IN: new time
 #endif
 {
    HgfsHandle fileHandle = HGFS_INVALID_HANDLE;
@@ -933,17 +979,10 @@ hgfs_utime(const char *path,        //IN: path to a file
    attr->mask = (HGFS_ATTR_VALID_WRITE_TIME |
                  HGFS_ATTR_VALID_ACCESS_TIME);
 
-#ifdef HAVE_UTIMENSAT
    accessTimeSec = ts[0].tv_sec;
    accessTimeNsec = ts[0].tv_nsec;
    writeTimeSec = ts[1].tv_sec;
    writeTimeNsec = ts[1].tv_nsec;
-#else
-   accessTimeSec = times->actime;
-   accessTimeNsec = 0;
-   writeTimeSec = times->modtime;
-   writeTimeNsec = 0;
-#endif
    attr->accessTime = HgfsConvertToNtTime(accessTimeSec, accessTimeNsec);
    attr->writeTime = HgfsConvertToNtTime(writeTimeSec, writeTimeNsec);
 
@@ -1136,7 +1175,14 @@ hgfs_write(const char *path,          //IN: path to a file
    }
 
    res = HgfsWrite(fi, buf, size, offset);
-   HgfsInvalidateAttrCache(abspath);
+   if (res >= 0) {
+      /*
+       * Positive result indicates the number of bytes written.
+       * For zero bytes and no error, we still purge the cache
+       * this could effect the attributes.
+       */
+      HgfsInvalidateAttrCache(abspath);
+   }
 
 exit:
    LOG(4, ("Exit(%d)\n", res));
@@ -1239,8 +1285,14 @@ exit:
  *----------------------------------------------------------------------
  */
 
+#if FUSE_MAJOR_VERSION == 3
+static void*
+hgfs_init(struct fuse_conn_info *conn, // IN: unused
+          struct fuse_config *cfg)     // IN/OUT: unused
+#else
 static void*
 hgfs_init(struct fuse_conn_info *conn) // IN: unused
+#endif
 {
    pthread_t purgeCacheThread;
    int dummy;
@@ -1292,12 +1344,19 @@ hgfs_destroy(void *data) // IN: unused
    LOG(4, ("Entry()\n"));
 
    res = HgfsDestroySession();
-   free(gState->basePath);
-
    if (res < 0) {
       LOG(4, ("Destroy session failed. error = %d\n", res));
    }
 
+   HgfsTransportExit();
+
+   free(gState->basePath);
+
+   if (gState->conf != NULL) {
+      g_key_file_free(gState->conf);
+      gState->conf = NULL;
+   }
+
    LOG(4, ("Exit()\n"));
 }
 
@@ -1318,11 +1377,7 @@ static struct fuse_operations vmhgfs_operations = {
    .chmod       = hgfs_chmod,
    .chown       = hgfs_chown,
    .truncate    = hgfs_truncate,
-#ifdef HAVE_UTIMENSAT
    .utimens     = hgfs_utimens,
-#else // HAVE_UTIMENSAT
-   .utime       = hgfs_utime,
-#endif // defined HAVE_UTIMENSAT
    .open        = hgfs_open,
    .read        = hgfs_read,
    .write       = hgfs_write,
@@ -1368,7 +1423,7 @@ main(int argc,       //IN: Argument count
    HgfsResetOps();
    res = HgfsTransportInit();
    if (res != 0) {
-      LOG(4, ("Main: Error in HgfsTransportInit %d\n", res));
+      fprintf(stderr, "Error %d cannot open connection!\n", res);
       return res;
    }
    HgfsInitCache();
